import shioaji as sj
from textual.app import App
from textual.widgets import ScrollView
from sjtop.dashboard import ContractDashBoard
from sjtop.side import ContractsTree, ContractClick

from sjtop.status_panel import StatusPanel


class SJTop(App):
    api: sj.Shioaji

    async def on_load(self) -> None:
        """Sent before going in to application mode."""

        # Bind our basic keys
        await self.bind("b", "view.toggle('sidebar')", "Toggle sidebar")
        await self.bind("q", "quit", "Quit")

    async def on_mount(self) -> None:
        """Call after terminal goes in to application mode"""
        self.api = sj.Shioaji(simulation=True)
        self.status_panel = StatusPanel("status_panel", "logining...")
        self.api.quote.set_event_callback(self.on_api_session_event)
        await self.view.dock(self.status_panel, edge="bottom", size=3)
        self.api.login("PAPIUSER01", "2222")
        self.tree = ContractsTree(self.api.Contracts, "contracts")
        self.side = ScrollView(self.tree, name="sidebar")
        await self.view.dock(self.side, edge="left", size=25)
        self.dashbaord = ContractDashBoard("dashboard", self.api)
        await self.view.dock(self.dashbaord, edge="right")

    def on_api_session_event(self, resp_code, event_code, info, event):
        self.status_panel.fit(
            f"Response Code: {resp_code} | Event Code: {event_code} | Info: {info} | Event: {event}"
        )

    async def handle_contract_click(self, message: ContractClick) -> None:
        """A message sent by the contract tree when a contract is clicked."""
        self.dashbaord.change_contract(message.contract)



from dataclasses import dataclass
from typing import Optional
import rich
from shioaji.constant import SecurityType

from shioaji.contracts import Contract, Contracts
from textual.message import Message
from textual._types import MessageTarget
from textual.widgets import TreeControl, TreeNode, TreeClick
from textual.reactive import Reactive
from textual.events import Mount


@dataclass
class ContractEntry:
    code: Optional[str]
    security_type: Optional[SecurityType]
    exchange: Optional[str]
    is_contracts: bool


@rich.repr.auto
class ContractClick(Message, bubble=True):
    def __init__(self, sender: MessageTarget, contract: Contract) -> None:
        self.contract = contract
        super().__init__(sender)


class ContractsTree(TreeControl[ContractEntry]):
    def __init__(self, contracts: Contracts, name: str = None) -> None:
        self.contract: Optional[Contract] = None
        self.contracts = contracts
        label = "Contracts"
        data = ContractEntry(
            label, security_type=None, exchange=None, is_contracts=True
        )
        super().__init__(label, name=name, data=data)
        self.root.tree.guide_style = "black"

    has_focus: Reactive[bool] = Reactive(False)

    def on_focus(self) -> None:
        self.has_focus = True

    def on_blur(self) -> None:
        self.has_focus = False

    async def on_mount(self, event: Mount) -> None:
        await self.load_contracts(self.root)

    async def load_contracts(self, node: TreeNode[ContractEntry]):
        if not node.data.security_type:
            for secu_type, cstream in self.contracts:
                await node.add(
                    secu_type,
                    ContractEntry(
                        code=None,
                        security_type=getattr(SecurityType, secu_type[:-1]),
                        exchange=None,
                        is_contracts=True,
                    ),
                )
        else:
            cstream = getattr(self.contracts, f"{node.data.security_type.name}s")
            if not node.data.exchange:
                for cs in cstream:
                    await node.add(
                        cs._name,
                        ContractEntry(
                            code=None,
                            security_type=node.data.security_type,
                            exchange=cs._name,
                            is_contracts=True,
                        ),
                    )
            else:
                cs = getattr(cstream, node.data.exchange)
                cs_sorted = sorted([c for c in cs], key=lambda x: x.code)
                for c in cs_sorted:
                    if node.data.security_type == SecurityType.Stock:
                        if len(c.code) == 4:
                            await node.add(
                                c.code,
                                ContractEntry(
                                    code=c.code,
                                    security_type=node.data.security_type,
                                    exchange=cs._name,
                                    is_contracts=False,
                                ),
                            )
                    else:
                        await node.add(
                            c.code,
                            ContractEntry(
                                code=c.code,
                                security_type=node.data.security_type,
                                exchange=cs._name,
                                is_contracts=False,
                            ),
                        )

        node.loaded = True
        await node.expand()
        self.refresh(layout=True)

    async def handle_tree_click(self, message: TreeClick[ContractEntry]) -> None:
        contract_entry = message.node.data
        if not contract_entry.is_contracts:
            contract = getattr(self.contracts, f"{contract_entry.security_type.name}s")[
                contract_entry.code
            ]
            await self.emit(ContractClick(self, contract))
        else:
            if not message.node.loaded:
                await self.load_contracts(message.node)
                await message.node.expand()
            else:
                await message.node.toggle()



from rich.panel import Panel
from textual.reactive import Reactive
from textual.widget import Widget


class StatusPanel(Widget):
    def __init__(self, name: str , status: str) -> None:
        self.status_panel = Panel(status, title="Session Event")
        super().__init__(name=name)

    mouse_over = Reactive(False)

    def fit(self, text: str):
        self.status_panel = Panel(text, title="Session Event")
        self.refresh()

    def render(self) -> Panel:
        return self.status_panel

    def on_enter(self) -> None:
        self.mouse_over = True

    def on_leave(self) -> None:
        self.mouse_over = False
        


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sjtop']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=0.24.1', 'shioaji>=0.3.3-alpha.3,<0.4.0', 'textual>=0.1.12,<0.2.0']

entry_points = \
{'console_scripts': ['sjtop = sjtop.__main__:main']}

setup_kwargs = {
    'name': 'sjtop',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'Yvictor',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import shioaji as sj
import pandas as pd
import numpy as np
from datetime import datetime
from decimal import Decimal

from shioaji.constant import QuoteVersion, QuoteType
from textual.widget import Widget
from rich.table import Table
from rich.bar import Bar
from rich.text import Text
from rich.console import Group


class ContractDashBoard(Widget):
    def __init__(self, name: str, api: sj.Shioaji) -> None:
        self.api = api
        super().__init__(name=name)

    def change_contract(self, contract: sj.contracts.Contract):
        self.api.quote.unsubscribe(
            self.contract, QuoteType.BidAsk, version=QuoteVersion.v1
        )
        self.api.quote.unsubscribe(
            self.contract, QuoteType.Tick, version=QuoteVersion.v1
        )
        self.contract = contract
        self.api.quote.subscribe(
            self.contract, QuoteType.BidAsk, version=QuoteVersion.v1
        )
        self.api.quote.subscribe(self.contract, QuoteType.Tick, version=QuoteVersion.v1)
        self.table.title = self.contract.symbol

    def on_mount(self):
        self.contract = min(
            [c for c in self.api.Contracts.Futures.TXF], key=lambda c: c.symbol
        )
        self.api.quote.set_on_tick_fop_v1_callback(self.on_fop_v1_tick)
        self.api.quote.set_on_bidask_fop_v1_callback(self.on_fop_v1_bidask)
        self.api.quote.subscribe(
            self.contract, QuoteType.BidAsk, version=QuoteVersion.v1
        )
        self.api.quote.subscribe(self.contract, QuoteType.Tick, version=QuoteVersion.v1)
        self.table = Table(
            title=self.contract.symbol,
            show_header=True,
            show_edge=False,
            pad_edge=False,
        )
        self.df = pd.DataFrame(
            np.zeros((10, 6)),
            columns=[
                "BidVolume",
                "BidPrice",
                "TickPrice",
                "TickVolume",
                "AskPrice",
                "AskVolume",
            ],
        )
        self.table.add_column("")
        for col in self.df.columns:
            self.table.add_column(col)
        # self.table.add_column("")
        self.bars = [
            Bar(100, 0, 0, color="green" if i < 5 else "red") for i in range(10)
        ]
        self.total_bar = Bar(100, 0, 50, color="green", bgcolor="red")
        self.deal_side_bar = Bar(1, 0.0, 0.5, color="red", bgcolor="green")

        self.cur_tick = sj.TickFOPv1(
            code=self.contract.code,
            datetime=datetime(2021, 1, 1, 0, 0, 0, 0),
            open=Decimal("0"),
            underlying_price=Decimal("0"),
            bid_side_total_vol=1,
            ask_side_total_vol=1,
            avg_price=Decimal("0"),
            close=Decimal("0"),
            high=Decimal("0"),
            low=Decimal("0"),
            amount=Decimal("0"),
            total_amount=Decimal("0"),
            volume=0,
            total_volume=0,
            tick_type=0,
            chg_type=0,
            price_chg=Decimal("0"),
            pct_chg=Decimal("0"),
            simtrade=1,
        )

        self.set_interval(0.0075, self.refresh)

    def on_fop_v1_tick(self, exchange: sj.Exchange, tick: sj.TickFOPv1):
        self.cur_tick = tick
        cond = (self.df.BidPrice == tick.close) | (self.df.AskPrice == tick.close)
        self.df.loc[~cond, "TickPrice"] = 0
        self.df.loc[~cond, "TickVolume"] = 0
        if cond.sum():
            self.df.loc[cond, "TickPrice"] = tick.close
            self.df.loc[cond, "TickVolume"] = tick.volume
        # self.refresh()

    def on_fop_v1_bidask(self, exchange: sj.Exchange, quote: sj.BidAskFOPv1):
        self.df.loc[0:4, "AskPrice"] = quote.ask_price[::-1]
        self.df.loc[0:4, "AskVolume"] = quote.ask_volume[::-1]
        self.df.loc[5:10, "BidPrice"] = quote.bid_price
        self.df.loc[5:10, "BidVolume"] = quote.bid_volume
        # self.refresh()

    def render(self):
        for col in self.table.columns:
            col._cells = []
        self.table.rows = []

        barsize = max(
            self.df["BidVolume"].cumsum().max(), self.df["AskVolume"].cumsum().max()
        )
        bidcumvol = self.df["BidVolume"].cumsum()
        askcumvol = self.df.loc[::-1, "AskVolume"].cumsum().loc[::-1]
        bidsum = self.df["BidVolume"].sum()
        asksum = self.df["AskVolume"].sum()
        self.total_bar.size = bidsum + asksum
        # self.total_bar.begin = 0
        self.total_bar.end = asksum

        for idx in range(10):
            bar = self.bars[idx]
            bar.size = barsize
            bar.begin = 0 if idx < 5 else barsize - bidcumvol[idx]
            bar.end = askcumvol[idx] if idx < 5 else barsize
            self.table.add_row(
                *[
                    bar,  # "" if idx < 5 else bar,
                    *self.df.loc[idx].map(lambda x: str(x) if x else "").tolist(),
                    # bar if idx < 5 else "",
                ],
                style="green" if idx < 5 else "red",
            )
        row_color = "green" if self.cur_tick.tick_type == 1 else "red"
        self.deal_side_bar.end = self.cur_tick.bid_side_total_vol / (
            self.cur_tick.ask_side_total_vol + self.cur_tick.bid_side_total_vol
        )
        self.table.add_row(
            *[
                self.total_bar,
                Text(str(bidsum), style="red"),
                "",
                Text(str(self.cur_tick.close), style=row_color),
                # Text(str(self.cur_tick.volume), style=row_color),
                Group(
                    Text(
                        f"{self.cur_tick.volume}          {self.deal_side_bar.end * 100:.4f}%",
                        style=row_color,
                    ),
                    self.deal_side_bar,
                ),
                "",
                Text(str(asksum), style="green"),
            ],
            # style="green" if idx < 5 else "red",
        )
        return self.table
